// SPDX-FileCopyrightText: 2021 MYNIJ
//
// SPDX-License-Identifier: GPL-3.0-or-later

var fs = require("fs");
var args = require("yargs")
  .usage("Usage : crawler.js -link website_link -f file_name")
  .demandOption(['link'])
  .demandOption(['file'])
  .alias("f", "file")
  .describe("file", "name of the file the results will be saved into")
  .alias("l", "link")
  .describe("link", "link to the website to be made into a site map")
  .alias("d", "depth")
  .describe("depth", "crawling depth")
  .nargs("link", 1)
  .nargs("file", 1)
  .nargs("depth", 1)
  .argv;
  
fs.open(args.file, 'w', function(err, file){
  if (err) throw err;
  }
);

var depth = 3,
  count = 1,
  link = args.link,
  builder = require("xmlbuilder"),
  readline = require('readline'),
  SCrawler = require("simplecrawler"),
  url_list = [],
  crawler = new SCrawler(link);

if (args.depth) depth = args.depth;

crawler.interval = 250;
crawler.maxConcurrency = 5;
crawler.maxDepth = depth;

crawler.on("fetchcomplete", function(queueItem, responseBuffer, response) {
  var mime = response.headers["content-type"];
  if (mime.startsWith("text/html") || mime.startsWith("text/plain")) {
    readline.cursorTo(process.stdout, 0);
    process.stdout.write(count + "");
    count+=1;
    url_list.push({
      "loc": queueItem.url
    });
  }
});

// Fire callback
crawler.on("complete", function() {
  readline.cursorTo(process.stdout, 0);
  console.log("crawled " + url_list.length + " urls");
  var xml_dict = {
    "urlset": {
      "@xmlns": "http://www.sitemaps.org/schemas/sitemap/0.9",
      "url": url_list
    }
  };

  var xml = builder.create(xml_dict, { encoding: 'utf-8' });
  fs.appendFile(args.file, xml.end({ pretty: true }), function(err){if(err) throw err;});
});

crawler.on("fetcherror", function(queueItem, response) {
  console.log("");
  console.log("Error " + response.statusCode + " while fetching " + queueItem.url);
});

// Start Crawl
crawler.start();


